@@ -219,6 +219,7 @@ struct sshfs {
 	int buflimit_workaround;
 	int unrel_append;
 	int fstat_workaround;
+	int createmode_workaround;
 	int transform_symlinks;
 	int follow_symlinks;
 	int no_check_root;
@@ -455,6 +456,8 @@ static struct fuse_opt workaround_opts[] = {
 	SSHFS_OPT("nobuflimit", buflimit_workaround, 0),
 	SSHFS_OPT("fstat",      fstat_workaround, 1),
 	SSHFS_OPT("nofstat",    fstat_workaround, 0),
+	SSHFS_OPT("createmode",   createmode_workaround, 1),
+	SSHFS_OPT("nocreatemode", createmode_workaround, 0),
 	FUSE_OPT_END
 };
 
@@ -2470,6 +2473,14 @@ static int sshfs_utimens(const char *path, const struct timespec tv[2],
 	int err;
 	struct buffer buf;
 	struct sshfs_file *sf = NULL;
+	time_t asec = tv[0].tv_sec, msec = tv[1].tv_sec;
+
+	struct timeval now;
+	gettimeofday(&now, NULL);
+	if (asec == 0)
+		asec = now.tv_sec;
+	if (msec == 0)
+		msec = now.tv_sec;
 
 	if (fi != NULL) {
 		sf = get_sshfs_file(fi);
@@ -2483,8 +2494,8 @@ static int sshfs_utimens(const char *path, const struct timespec tv[2],
 	else 
 		buf_add_buf(&buf, &sf->handle);
 	buf_add_uint32(&buf, SSH_FILEXFER_ATTR_ACMODTIME);
-	buf_add_uint32(&buf, tv[0].tv_sec);
-	buf_add_uint32(&buf, tv[1].tv_sec);
+	buf_add_uint32(&buf, asec);
+	buf_add_uint32(&buf, msec);
 
 	err = sftp_request(sf == NULL ? SSH_FXP_SETSTAT : SSH_FXP_FSETSTAT,
 			   &buf, SSH_FXP_STATUS, NULL);
@@ -3100,6 +3111,9 @@ static int sshfs_statfs(const char *path, struct statvfs *buf)
 static int sshfs_create(const char *path, mode_t mode,
                         struct fuse_file_info *fi)
 {
+	if (sshfs.createmode_workaround)
+		mode = 0;
+
 	return sshfs_open_common(path, mode, fi);
 }
 
@@ -3144,7 +3158,7 @@ static int sshfs_getattr(const char *path, struct stat *stbuf,
 	struct buffer outbuf;
 	struct sshfs_file *sf = NULL;
 
-	if (fi != NULL || sshfs.fstat_workaround) {
+	if (fi != NULL && !sshfs.fstat_workaround) {
 		sf = get_sshfs_file(fi);
 		if (!sshfs_file_is_conn(sf))
 			return -EIO;
@@ -3374,6 +3388,7 @@ static void usage(const char *progname)
 "             [no]truncate     fix truncate for old servers (default: off)\n"
 "             [no]buflimit     fix buffer fillup bug in server (default: on)\n"
 "             [no]fstat        always use stat() instead of fstat() (default: off)\n"
+"             [no]createmode   always pass mode 0 to create (default: off)\n"
 "    -o idmap=TYPE          user/group ID mapping (default: " IDMAP_DEFAULT ")\n"
 "             none             no translation of the ID space\n"
 "             user             only translate UID/GID of connecting user\n"
@@ -3860,6 +3875,7 @@ int main(int argc, char *argv[])
 	sshfs.renamexdev_workaround = 0;
 	sshfs.truncate_workaround = 0;
 	sshfs.buflimit_workaround = 1;
+	sshfs.createmode_workaround = 0;
 	sshfs.ssh_ver = 2;
 	sshfs.progname = argv[0];
 	sshfs.rfd = -1;
